"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Buffer_1 = require("./Buffer");
var Hasher = /** @class */ (function () {
    function Hasher(tbl) {
        this.tbl = tbl;
        this.crc = 0xffffffff;
    }
    Hasher.prototype.append = function (data) {
        var tbl = this.tbl;
        var crc = this.crc;
        if (typeof (data) == 'string')
            data = Buffer_1.encodeArray(data);
        var len = data.length;
        var pos = 0;
        for (var pos_1 = 0; pos_1 < len; ++pos_1) {
            crc = (crc >>> 8) ^ tbl[(crc & 0xff) ^ data[pos_1]];
        }
        this.crc = crc;
        return ((crc ^ 0xffffffff) >>> 0);
    };
    return Hasher;
}());
/** 32-bit Cyclic Redundancy Check. */
var CRC32 = /** @class */ (function () {
    /** @param poly Reversed generator polynomial, default edb88320 (Ethernet, GZIP, PNG).
      * Other good choices are 82f63b78 (Castagnoli) used in Btrfs and eb31d82e (Koopman). */
    function CRC32(poly) {
        if (poly === void 0) { poly = 0xedb88320; }
        this.poly = poly;
        this.tbl = [];
        for (var n = 0; n < 256; ++n) {
            var crc = n;
            var b = 8;
            while (b--) {
                crc = ((crc >>> 1) ^ (-(crc & 1) & poly)) >>> 0;
            }
            this.tbl[n] = crc;
        }
    }
    CRC32.prototype.create = function () {
        return (new Hasher(this.tbl));
    };
    return CRC32;
}());
exports.CRC32 = CRC32;
