"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Namespace_1 = require("../Namespace");
var SimpleSchema_1 = require("../schema/SimpleSchema");
var RuleSet_1 = require("./RuleSet");
var ComplexType_1 = require("../schema/ComplexType");
var Element_1 = require("../schema/Element");
var Builder = /** @class */ (function () {
    function Builder(parserConfig, schemaSpec) {
        this.ruleSetTbl = {};
        this.unknownType = new ComplexType_1.ComplexType();
        this.options = parserConfig.options;
        for (var _i = 0, _a = Object.keys(schemaSpec); _i < _a.length; _i++) {
            var prefix = _a[_i];
            var _b = schemaSpec[prefix], defaultPrefix = _b[0], nsUri = _b[1], spec = _b[2];
            var ns = new Namespace_1.Namespace(defaultPrefix, nsUri);
            if (spec['document']) {
                this.ruleSetTbl[nsUri] = new RuleSet_1.RuleSet(new SimpleSchema_1.SimpleSchema(parserConfig, ns, spec));
            }
        }
    }
    Builder.prototype.getUnknownProto = function (token) {
        var elementSpec = this.unknownType.elements && this.unknownType.elements.group.tbl[token.id];
        if (!elementSpec) {
            elementSpec = new Element_1.ElementSpec(0, Infinity);
            var elementMeta = new Element_1.ElementMeta(token);
            elementMeta.type = new ComplexType_1.ComplexType();
            elementSpec.meta = elementMeta;
            this.unknownType.addAll(elementSpec);
        }
        return (elementSpec.meta.createProto());
    };
    Builder.prototype.build = function (parser, nsUri, cb) {
        var _this = this;
        var document = {};
        var stackPos = 0;
        var rule = this.ruleSetTbl[nsUri].rootRule;
        var ruleNext;
        var member;
        var ruleStack = [];
        var item = document;
        var itemNext;
        var itemStack = [];
        var ignoreDepth = 0;
        var state = 2 /* TEXT */;
        var target;
        parser.on('data', function (chunk) {
            if (!chunk)
                return;
            var buffer = chunk.buffer;
            var token;
            var dataType;
            var kind;
            var id;
            var name;
            var lastNum = chunk.length - 1;
            var tokenNum = -1;
            while (tokenNum < lastNum) {
                token = buffer[++tokenNum];
                dataType = typeof (token);
                if (ignoreDepth) {
                    if (dataType == 'object') {
                        kind = token.kind;
                        if (kind == 0 /* open */)
                            ++ignoreDepth;
                        else if (kind == 1 /* close */)
                            --ignoreDepth;
                    }
                }
                else if (dataType == 'object') {
                    kind = token.kind;
                    switch (kind) {
                        case 0 /* open */:
                            id = token.id;
                            name = token.name;
                            member = rule && rule.elements[id];
                            if (member) {
                                ruleNext = member.rule;
                                if (ruleNext == RuleSet_1.Rule.string) {
                                    // NOTE: If the string element has attributes,
                                    // they're added to its parent element!
                                    target = name;
                                    itemNext = item;
                                }
                                else {
                                    itemNext = new ruleNext.XMLType();
                                    if (member.max > 1) {
                                        if (!item.hasOwnProperty(name))
                                            item[name] = [];
                                        item[name].push(itemNext);
                                    }
                                    else
                                        item[name] = itemNext;
                                }
                            }
                            else if (!_this.options.parseUnknown) {
                                ++ignoreDepth;
                                state = 2 /* TEXT */;
                                break;
                            }
                            else {
                                ruleNext = void 0;
                                itemNext = new (_this.getUnknownProto(token))();
                                if (!item.hasOwnProperty(name))
                                    item[name] = itemNext;
                                else if (item[name] instanceof Array)
                                    item[name].push(itemNext);
                                else
                                    item[name] = [item[name], itemNext];
                            }
                            itemStack[stackPos] = item;
                            ruleStack[stackPos++] = rule;
                            item = itemNext;
                            rule = ruleNext;
                            state = 0 /* ELEMENT */;
                            break;
                        case 1 /* close */:
                            item = itemStack[--stackPos];
                            rule = ruleStack[stackPos];
                        // Fallthru
                        case 2 /* emitted */:
                            if (rule != RuleSet_1.Rule.string)
                                target = '$';
                            state = 2 /* TEXT */;
                            break;
                        case 3 /* string */:
                            id = token.id;
                            member = rule && rule.attributes[id];
                            if (member || _this.options.parseUnknown) {
                                target = token.name;
                            }
                            else {
                                target = void 0;
                            }
                            break;
                        case 5 /* comment */:
                            state = 3 /* COMMENT */;
                            break;
                    }
                }
                else {
                    switch (state) {
                        case 2 /* TEXT */:
                        case 0 /* ELEMENT */:
                            if (target) {
                                item[target] = (member && member.max > 1) ? (token + '').split(/ +/) : token;
                                target = void 0;
                            }
                            break;
                    }
                }
            }
            chunk.free();
        });
        parser.on('end', function () { return cb(null, document); });
    };
    return Builder;
}());
exports.Builder = Builder;
