"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Element_1 = require("../schema/Element");
var Rule = /** @class */ (function () {
    function Rule() {
        this.elements = {};
        this.attributes = {};
    }
    Rule.prototype.addElement = function (member) {
        this.elements[member.id] = member;
    };
    Rule.prototype.addAttribute = function (member) {
        this.attributes[member.id] = member;
    };
    Rule.string = new Rule();
    return Rule;
}());
exports.Rule = Rule;
var RuleMember = /** @class */ (function () {
    function RuleMember(rule, spec) {
        this.rule = rule;
        this.spec = spec;
        this.id = spec.meta.token.id;
        this.min = spec.min;
        this.max = spec.max;
    }
    return RuleMember;
}());
exports.RuleMember = RuleMember;
function link(parent) {
    function Result() { }
    Result.prototype = parent;
    return (new Result());
}
var RuleSet = /** @class */ (function () {
    function RuleSet(schema) {
        this.schema = schema;
        this.rootRule = this.createRule(schema.document);
    }
    RuleSet.prototype.createRule = function (type, meta, parent) {
        var rule = new Rule();
        var childRule;
        var proto = {};
        if (meta) {
            rule.XMLType = meta.createProto();
            proto = rule.XMLType.prototype;
        }
        if (type.elements && type.elements.group) {
            for (var _i = 0, _a = type.elements.group.list; _i < _a.length; _i++) {
                var childSpec = _a[_i];
                var memberMeta = childSpec.meta;
                if (memberMeta) {
                    if (memberMeta instanceof Element_1.ElementMeta) {
                        childRule = void 0;
                        for (var item = parent; item; item = item.parent) {
                            if (item.meta == memberMeta) {
                                // If the child element type matches an ancestor's,
                                // re-use its rule to avoid infinite recursion.
                                childRule = item.rule;
                                break;
                            }
                        }
                        if (!childRule) {
                            childRule = this.createRule(memberMeta.type, memberMeta, { meta: meta, rule: rule, parent: parent });
                        }
                        // Subclass type metadata and clear existence flag to indicate a placeholder.
                        var fakeMeta = link(memberMeta);
                        fakeMeta.exists = false;
                        var placeholder = new childRule.XMLType();
                        placeholder._ = fakeMeta;
                        memberMeta.placeholder = placeholder;
                        if (childSpec.max > 1) {
                            // Use arrays as placeholders for arrays of children.
                            placeholder = childSpec.min > 0 ? [placeholder] : [];
                        }
                        else if (childSpec.min < 1) {
                            placeholder = null;
                        }
                        if (placeholder) {
                            Object.defineProperty(proto, memberMeta.token.name, {
                                configurable: true,
                                enumerable: false,
                                value: placeholder,
                                writable: true
                            });
                        }
                    }
                    else
                        childRule = Rule.string;
                    rule.addElement(new RuleMember(childRule, childSpec));
                }
            }
        }
        if (type.attributes) {
            for (var _b = 0, _c = type.attributes.list; _b < _c.length; _b++) {
                var attributeSpec = _c[_b];
                var memberMeta = attributeSpec.meta;
                if (memberMeta) {
                    // const token = memberMeta.token;
                    childRule = Rule.string;
                    rule.addAttribute(new RuleMember(childRule, attributeSpec));
                }
            }
        }
        return (rule);
    };
    return RuleSet;
}());
exports.RuleSet = RuleSet;
