/// <reference types="node" />
import * as stream from 'stream';
import { ArrayType } from '../Buffer';
import { ErrorType } from '../tokenizer/ErrorType';
import { NativeParser } from './ParserLib';
import { ParserConfig } from './ParserConfig';
import { InternalToken } from './InternalToken';
import { TokenChunk } from './TokenChunk';
export declare class ParseError extends Error {
    code: ErrorType;
    row: number;
    col: number;
    constructor(code: ErrorType, row: number, col: number);
}
/** XML parser stream, emits tokens with fully qualified names. */
export declare class Parser extends stream.Transform {
    private config;
    private native;
    /** Call only from ParserConfig.createParser.
      * @param config Reference to C++ config object.
      * @param native Reference to C++ parser object. */
    constructor(config: ParserConfig, native: NativeParser);
    parseSync(data: string | ArrayType): TokenChunk;
    getConfig(): ParserConfig;
    bindPrefix(prefix: InternalToken, uri: InternalToken): void;
    private throwError(msg, row, col);
    _flush(flush: (err: any, chunk: TokenChunk | null) => void): void;
    _transform(chunk: string | ArrayType, enc: string, flush: (err: any, chunk: TokenChunk | null) => void): void;
    private parseCodeBuffer(pending);
    private storeSlice(start, end?);
    /** getSlice helper for concatenating buffer parts. */
    private buildSlice(start, end?);
    /** Get a string from the input buffer. Prepend any parts left from
      * previous code buffers. */
    private getSlice(start, end?);
    /** Resolve any prior occurrences of a recently defined prefix
      * within the same element. */
    private resolve(elementStart, tokenNum, prefix, idNamespace);
    /** Current element not yet emitted (closing angle bracket unseen). */
    private latestElement;
    /** Previous namespace prefix token, applied to the next element, attribute
      * or xmlns definition. */
    private latestPrefix;
    private latestNamespace;
    /** Current input buffer. */
    private chunk;
    private namespaceList;
    private namespacesChanged;
    /** Storage for parts of strings split between chunks of input. */
    private partList;
    private partListTotalByteLen;
    /** Offset to start of text in input buffer, or -1 if not reading text. */
    private partStart;
    /** Number of valid initial bytes in next token. */
    private partialLen;
    /** Shared with C++ library. */
    private codeBuffer;
    /** Stream output buffer chunk. */
    tokenChunk: TokenChunk;
    /** Offset to start of current element definition in output buffer. */
    private elementStart;
    /** Prefixes of latest tokenBuffer entries (their namespace may change
      * if the prefix is remapped). Index 0 corresponds to elementStart. */
    private prefixBuffer;
    private namespaceBuffer;
    /** Unresolved elements (temporary tokens lacking a namespace). */
    private unknownElementTbl;
    /** Unresolved attributes (temporary tokens lacking a namespace). */
    private unknownAttributeTbl;
    private unknownOffsetList;
    private unknownCount;
    private hasError;
}
