"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var stream = require("stream");
var Buffer_1 = require("../Buffer");
var Namespace_1 = require("../Namespace");
var TokenChunk_1 = require("./TokenChunk");
var Token_1 = require("./Token");
// const codeBufferSize = 2;
// const codeBufferSize = 3;
var codeBufferSize = 8192;
var chunkSize = Infinity;
var ParseError = /** @class */ (function (_super) {
    __extends(ParseError, _super);
    function ParseError(code, row, col) {
        var _this = _super.call(this, 'Parse error on line ' + row + ' column ' + col) || this;
        _this.code = code;
        _this.row = row;
        _this.col = col;
        return _this;
    }
    return ParseError;
}(Error));
exports.ParseError = ParseError;
/** XML parser stream, emits tokens with fully qualified names. */
var Parser = /** @class */ (function (_super) {
    __extends(Parser, _super);
    /** Call only from ParserConfig.createParser.
      * @param config Reference to C++ config object.
      * @param native Reference to C++ parser object. */
    function Parser(config, native) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.config = config;
        _this.native = native;
        _this.namespaceList = [];
        _this.namespacesChanged = true;
        /** Storage for parts of strings split between chunks of input. */
        _this.partList = null;
        _this.partListTotalByteLen = 0;
        /** Offset to start of text in input buffer, or -1 if not reading text. */
        _this.partStart = -1;
        /** Stream output buffer chunk. */
        _this.tokenChunk = TokenChunk_1.TokenChunk.allocate();
        /** Offset to start of current element definition in output buffer. */
        _this.elementStart = -1;
        /** Prefixes of latest tokenBuffer entries (their namespace may change
          * if the prefix is remapped). Index 0 corresponds to elementStart. */
        _this.prefixBuffer = [];
        _this.namespaceBuffer = [];
        /** Unresolved elements (temporary tokens lacking a namespace). */
        _this.unknownElementTbl = {};
        /** Unresolved attributes (temporary tokens lacking a namespace). */
        _this.unknownAttributeTbl = {};
        _this.unknownOffsetList = [];
        _this.unknownCount = 0;
        _this.hasError = false;
        _this.codeBuffer = new Uint32Array(codeBufferSize);
        _this.native.setCodeBuffer(_this.codeBuffer, function () { return _this.parseCodeBuffer(true); });
        for (var _i = 0, _a = _this.config.namespaceList; _i < _a.length; _i++) {
            var ns = _a[_i];
            if (ns && (ns.base.isSpecial || ns.base.defaultPrefix == 'xml')) {
                _this.namespaceList[ns.base.id] = ns.base;
            }
        }
        return _this;
    }
    Parser.prototype.parseSync = function (data) {
        var buffer = [];
        var namespaceList;
        this.on('data', function (chunk) {
            for (var _i = 0, _a = chunk.buffer; _i < _a.length; _i++) {
                var token = _a[_i];
                buffer.push(token);
            }
            if (chunk.namespaceList)
                namespaceList = chunk.namespaceList;
            chunk.free();
        });
        this.on('error', function (err) { throw (err); });
        this.write(data);
        var output = TokenChunk_1.TokenChunk.allocate(buffer);
        output.namespaceList = namespaceList;
        return (output);
    };
    Parser.prototype.getConfig = function () { return (this.config); };
    Parser.prototype.bindPrefix = function (prefix, uri) {
        this.native.bindPrefix(prefix.id, uri.id);
    };
    Parser.prototype.throwError = function (msg, row, col) {
        var err = new ParseError(msg, row + 1, col + 1);
        this.emit('error', err);
        this.hasError = true;
    };
    Parser.prototype._flush = function (flush) {
        this.native.destroy();
        flush(null, null);
    };
    Parser.prototype._transform = function (chunk, enc, flush) {
        if (this.hasError)
            return;
        if (typeof (chunk) == 'string')
            chunk = Buffer_1.encodeArray(chunk);
        var len = chunk.length;
        var nativeStatus = 0 /* OK */;
        var next;
        if (len < chunkSize) {
            this.chunk = chunk;
            nativeStatus = this.native.parse(this.chunk);
            this.parseCodeBuffer(false);
        }
        else {
            // Limit size of buffers sent to native code.
            for (var pos = 0; pos < len; pos = next) {
                next = Math.min(pos + chunkSize, len);
                this.chunk = chunk.slice(pos, next);
                nativeStatus = this.native.parse(this.chunk);
                if (nativeStatus != 0 /* OK */)
                    break;
                this.parseCodeBuffer(false);
            }
        }
        if (nativeStatus != 0 /* OK */) {
            this.throwError(nativeStatus, this.native.row, this.native.col);
            return;
        }
        if (this.elementStart < 0) {
            if (this.namespacesChanged)
                this.tokenChunk.namespaceList = this.namespaceList;
            flush(null, this.tokenChunk);
            this.tokenChunk = TokenChunk_1.TokenChunk.allocate();
        }
        else {
            // Not ready to flush but have to send something to get more input.
            flush(null, null);
        }
    };
    Parser.prototype.parseCodeBuffer = function (pending) {
        var config = this.config;
        var codeBuffer = this.codeBuffer;
        var codeCount = codeBuffer[0];
        // NOTE: These must be updated if config is unlinked!
        var elementList = config.elementSpace.list;
        var attributeList = config.attributeSpace.list;
        var prefixList = config.prefixSpace.list;
        var uriList = config.uriSpace.list;
        var partialList = elementList;
        var codeNum = 0;
        var partStart = this.partStart;
        var partialLen = this.partialLen;
        var latestElement = this.latestElement;
        var latestPrefix = this.latestPrefix;
        var latestNamespace = this.latestNamespace;
        var tokenBuffer = this.tokenChunk.buffer;
        var prefixBuffer = this.prefixBuffer;
        var namespaceBuffer = this.namespaceBuffer;
        var unknownElementTbl = this.unknownElementTbl;
        var unknownAttributeTbl = this.unknownAttributeTbl;
        var unknownOffsetList = this.unknownOffsetList;
        var tokenNum = this.tokenChunk.length - 1;
        var token;
        var linkTbl;
        var linkKind;
        var name;
        var elementStart = this.elementStart;
        var unknownCount = this.unknownCount;
        while (codeNum < codeCount) {
            var code = codeBuffer[++codeNum];
            var kind = code & 31 /* MASK */;
            code >>= 5 /* SHIFT */;
            switch (kind) {
                case 0 /* OPEN_ELEMENT_ID */:
                    latestElement = elementList[code].open;
                    // TODO: If latestprefix is null, use current prefix for element's namespace.
                    tokenBuffer[++tokenNum] = latestElement;
                    prefixBuffer[0] = latestPrefix;
                    elementStart = tokenNum;
                    break;
                case 1 /* CLOSE_ELEMENT_ID */:
                    tokenBuffer[++tokenNum] = elementList[code].close;
                    break;
                case 6 /* ELEMENT_EMITTED */:
                case 7 /* CLOSED_ELEMENT_EMITTED */:
                    if (unknownCount) {
                        var ns = void 0;
                        var offset = void 0;
                        for (var pos_1 = 0; pos_1 < unknownCount; ++pos_1) {
                            offset = unknownOffsetList[pos_1];
                            ns = namespaceBuffer[offset];
                            // If an xmlns definition already resolved
                            // this token, ns will be null.
                            if (ns) {
                                // Ensure namespace is updated after config unlink.
                                ns = config.namespaceList[ns.id];
                                tokenBuffer[offset + elementStart] = tokenBuffer[offset + elementStart].resolve(ns);
                            }
                        }
                        latestElement = tokenBuffer[elementStart];
                        unknownCount = 0;
                    }
                    tokenBuffer[++tokenNum] = (kind == 6 /* ELEMENT_EMITTED */ ?
                        latestElement.emitted :
                        latestElement.close);
                    elementStart = -1;
                    break;
                case 2 /* ATTRIBUTE_ID */:
                    tokenBuffer[++tokenNum] = attributeList[code].string;
                    // If latestprefix is null, set attribute prefix to match its parent element.
                    prefixBuffer[tokenNum - elementStart] = latestPrefix || prefixBuffer[0];
                    break;
                case 3 /* PREFIX_ID */:
                    latestNamespace = config.namespaceList[code >> 14];
                    code = code & 0x3fff;
                // Fallthru
                case 4 /* XMLNS_ID */:
                    latestPrefix = prefixList[code];
                    break;
                case 8 /* NAMESPACE_ID */:
                    this.resolve(elementStart, tokenNum, latestPrefix, code);
                    latestPrefix = null;
                    break;
                case 11 /* TEXT_START_OFFSET */:
                case 9 /* VALUE_START_OFFSET */:
                case 13 /* COMMENT_START_OFFSET */:
                case 15 /* UNKNOWN_START_OFFSET */:
                    partStart = code;
                    break;
                case 16 /* UNKNOWN_OPEN_ELEMENT_END_OFFSET */:
                    name = this.getSlice(partStart, code);
                    latestElement = unknownElementTbl[name];
                    if (!latestElement) {
                        latestElement = new Token_1.OpenToken(name, Namespace_1.Namespace.unknown);
                        unknownElementTbl[name] = latestElement;
                    }
                    tokenBuffer[++tokenNum] = latestElement;
                    prefixBuffer[0] = latestPrefix;
                    namespaceBuffer[0] = latestNamespace;
                    elementStart = tokenNum;
                    unknownOffsetList[0] = 0;
                    unknownCount = 1;
                    partStart = -1;
                    break;
                case 17 /* UNKNOWN_CLOSE_ELEMENT_END_OFFSET */:
                    name = this.getSlice(partStart, code);
                    tokenBuffer[++tokenNum] = (latestNamespace ?
                        latestNamespace.addElement(name) :
                        unknownElementTbl[name]).close;
                    partStart = -1;
                    break;
                case 18 /* UNKNOWN_ATTRIBUTE_END_OFFSET */:
                    name = this.getSlice(partStart, code);
                    token = unknownAttributeTbl[name];
                    if (!token) {
                        token = new Token_1.StringToken(name, Namespace_1.Namespace.unknown);
                        unknownAttributeTbl[name] = token;
                    }
                    tokenBuffer[++tokenNum] = token;
                    var pos = tokenNum - elementStart;
                    prefixBuffer[pos] = latestPrefix;
                    namespaceBuffer[pos] = latestNamespace;
                    unknownOffsetList[unknownCount++] = pos;
                    partStart = -1;
                    break;
                case 10 /* VALUE_END_OFFSET */:
                case 12 /* TEXT_END_OFFSET */:
                    tokenBuffer[++tokenNum] = this.getSlice(partStart, code);
                    partStart = -1;
                    break;
                case 19 /* UNKNOWN_PREFIX_END_OFFSET */:
                case 20 /* UNKNOWN_XMLNS_END_OFFSET */:
                case 21 /* UNKNOWN_URI_END_OFFSET */:
                    // Add the namespace prefix or URI to a separate trie.
                    // Incoming code buffer should have been flushed immediately
                    // after writing this token.
                    if (kind == 21 /* UNKNOWN_URI_END_OFFSET */) {
                        var uri = this.getSlice(partStart, code);
                        /* if(uri.id > dynamicTokenTblSize) {
                            // TODO: report row and column in error messages.
                            throw(new Error('Too many different xmlns URIs'));
                        } */
                        // Create a new namespace for the unrecognized URI.
                        name = latestPrefix.name;
                        var ns = new Namespace_1.Namespace(name, uri, config.maxNamespace + 1);
                        // This may unlink the config:
                        var idNamespace = config.bindNamespace(ns, latestPrefix.name, this);
                        this.resolve(elementStart, tokenNum, latestPrefix, idNamespace);
                        latestPrefix = null;
                    }
                    else {
                        // This may unlink the config:
                        latestPrefix = config.addPrefix(this.getSlice(partStart, code));
                        /* if(latestPrefix.id > dynamicTokenTblSize) {
                            // TODO: report row and column in error messages.
                            throw(new Error('Too many different xmlns prefixes'));
                        } */
                        this.native.setPrefix(latestPrefix.id);
                    }
                    // Config may have been unlinked so update references to it.
                    elementList = config.elementSpace.list;
                    attributeList = config.attributeSpace.list;
                    prefixList = config.prefixSpace.list;
                    uriList = config.uriSpace.list;
                    partStart = -1;
                    break;
                case 14 /* COMMENT_END_OFFSET */:
                    tokenBuffer[++tokenNum] = Token_1.SpecialToken.comment;
                    tokenBuffer[++tokenNum] = this.getSlice(partStart, code);
                    partStart = -1;
                    break;
                case 26 /* PARTIAL_LEN */:
                    partialLen = code;
                    break;
                case 25 /* PARTIAL_URI_ID */:
                    partialList = uriList;
                // Fallthru
                case 24 /* PARTIAL_PREFIX_ID */:
                    if (partialList == elementList)
                        partialList = prefixList;
                // Fallthru
                case 23 /* PARTIAL_ATTRIBUTE_ID */:
                    if (partialList == elementList)
                        partialList = attributeList;
                // Fallthru
                case 22 /* PARTIAL_ELEMENT_ID */:
                    this.partList = [partialList[code].buf.slice(0, partialLen)];
                    this.partListTotalByteLen = partialLen;
                    partialList = elementList;
                    break;
                default:
                    break;
            }
        }
        if (!pending && partStart >= 0) {
            this.storeSlice(partStart);
            partStart = 0;
        }
        // NOTE: Any active cursor in native code will still use the old trie
        // after update.
        config.updateNamespaces();
        this.partStart = partStart;
        this.partialLen = partialLen;
        this.latestElement = latestElement;
        this.latestPrefix = latestPrefix;
        this.latestNamespace = latestNamespace;
        this.tokenChunk.length = tokenNum + 1;
        this.elementStart = elementStart;
        this.unknownCount = unknownCount;
    };
    Parser.prototype.storeSlice = function (start, end) {
        if (!this.partList)
            this.partList = [];
        if (end !== 0) {
            this.partList.push(this.chunk.slice(start, end));
            this.partListTotalByteLen += (end || this.chunk.length) - start;
        }
    };
    /** getSlice helper for concatenating buffer parts. */
    Parser.prototype.buildSlice = function (start, end) {
        this.storeSlice(start, end);
        var result = Buffer_1.decodeArray(Buffer_1.concatArray(this.partList, this.partListTotalByteLen));
        this.partList = null;
        this.partListTotalByteLen = 0;
        return (result);
    };
    /** Get a string from the input buffer. Prepend any parts left from
      * previous code buffers. */
    Parser.prototype.getSlice = function (start, end) {
        return ((this.partList ? this.buildSlice(start, end) :
            Buffer_1.decodeArray(this.chunk, start, end)).replace(/\r\n?|\n\r/g, '\n'));
    };
    /** Resolve any prior occurrences of a recently defined prefix
      * within the same element. */
    Parser.prototype.resolve = function (elementStart, tokenNum, prefix, idNamespace) {
        var prefixBuffer = this.prefixBuffer;
        var tokenBuffer = this.tokenChunk.buffer;
        var ns = this.config.namespaceList[idNamespace];
        var len = tokenNum - elementStart;
        var token;
        if (!ns.base.defaultPrefix) {
            ns.base.defaultPrefix = prefix.name;
        }
        this.namespaceList[ns.base.id] = ns.base;
        this.namespacesChanged = true;
        for (var pos = 0; pos <= len; ++pos) {
            if (prefixBuffer[pos] == prefix) {
                token = tokenBuffer[pos + elementStart];
                if (token instanceof Token_1.MemberToken) {
                    tokenBuffer[pos + elementStart] = token.resolve(ns);
                    this.namespaceBuffer[pos] = null;
                }
            }
        }
    };
    return Parser;
}(stream.Transform));
exports.Parser = Parser;
