import { NativeConfig } from './ParserLib';
import { Namespace } from '../Namespace';
import { ParserNamespace } from './ParserNamespace';
import { TokenSpace } from '../tokenizer/TokenSpace';
import { TokenSet } from '../tokenizer/TokenSet';
import { InternalToken } from './InternalToken';
import { TokenChunk } from './TokenChunk';
import { MemberToken, OpenToken, CloseToken, EmittedToken, StringToken } from './Token';
import { Parser } from './Parser';
import { XModuleTable } from './JSX';
export interface ParserOptions {
    parseUnknown?: boolean;
    omitDefaults?: boolean;
}
export interface TokenTbl {
    [prefix: string]: {
        uri: string;
        elements?: string[];
        attributes?: string[];
    };
}
export interface Registry {
    tokens: {
        [name: string]: MemberToken;
    };
    elements: {
        [id: number]: string;
    };
    attributes: {
        [id: number]: string;
    };
}
/** Parser configuration for quickly instantiating new parsers.
  * Each parser instance holds a new, cloned copy. */
export declare class ParserConfig {
    /** XML parser configuration.
      * @param config Parent object for cloning.
      * @param native Reference to C++ object. For internal use only. */
    constructor(config?: ParserOptions | ParserConfig, native?: NativeConfig | null);
    unlink(): void;
    createParser(): Parser;
    parseSync(data: string): TokenChunk;
    getNamespace(uri: string): Namespace;
    addNamespace(nsBase: Namespace): number;
    bindNamespace(nsBase: Namespace, prefix?: string, parser?: Parser): number;
    updateNamespaces(): void;
    bindPrefix(prefix: InternalToken, uri: InternalToken): void;
    addUri(uri: string, ns: ParserNamespace): InternalToken;
    addPrefix(prefix: string): InternalToken;
    registerTokens(tbl: TokenTbl): Registry;
    jsxRegister<Module extends XModuleTable>(spec: Module): Module;
    jsxRegister<Module extends XModuleTable, Result>(spec: Module, handler?: (result: Module) => Result): Result;
    getElementTokens(ns: Namespace, name: string): [OpenToken | null, CloseToken | null, EmittedToken | null, StringToken | null];
    getAttributeTokens(ns: Namespace, name: string): [OpenToken | null, CloseToken | null, EmittedToken | null, StringToken | null];
    /** If true, object is a clone sharing data with a parent object. */
    private isLinked;
    /** If true, another object is a clone sharing data with this object. */
    private isFrozen;
    /** Reference to C++ object. */
    private native;
    options: ParserOptions;
    xmlnsToken: InternalToken;
    emptyPrefixToken: InternalToken;
    xmlnsPrefixToken: InternalToken;
    processingPrefixToken: InternalToken;
    /** Allocates ID numbers for xmlns uri tokens. */
    uriSpace: TokenSpace;
    /** Allocates ID numbers for xmlns prefix tokens. */
    prefixSpace: TokenSpace;
    /** Allocates ID numbers for element name tokens. */
    elementSpace: TokenSpace;
    /** Allocates ID numbers for attribute name tokens. */
    attributeSpace: TokenSpace;
    uriSet: TokenSet;
    prefixSet: TokenSet;
    /** List of supported namespaces. */
    namespaceList: ParserNamespace[];
    /** Mapping from URI to namespace. */
    private namespaceTbl;
    maxNamespace: number;
    registry: Registry;
    nsMapper?: (uri: string) => string | null | false | undefined;
}
