"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ParserLib_1 = require("./ParserLib");
var TokenSet_1 = require("../tokenizer/TokenSet");
var ParserNamespace = /** @class */ (function () {
    /** @param base Parser-independent namespace definition. */
    function ParserNamespace(parent, config) {
        this.parent = parent;
        if (parent instanceof ParserNamespace) {
            this.base = parent.base;
            this.native = parent.native.clone();
            this.elementSet = new TokenSet_1.TokenSet(config.elementSpace, parent.elementSet);
            this.attributeSet = new TokenSet_1.TokenSet(config.attributeSpace, parent.attributeSet);
        }
        else {
            this.base = parent;
            this.native = new ParserLib_1.NativeNamespace(parent.uri);
            this.elementSet = new TokenSet_1.TokenSet(config.elementSpace);
            this.attributeSet = new TokenSet_1.TokenSet(config.attributeSpace);
            this.attributeSet.addToken(config.xmlnsToken);
            for (var _i = 0, _a = parent.elementNameList; _i < _a.length; _i++) {
                var name_1 = _a[_i];
                this.addElement(name_1);
            }
            for (var _b = 0, _c = parent.attributeNameList; _b < _c.length; _b++) {
                var name_2 = _c[_b];
                this.addAttribute(name_2);
            }
        }
    }
    ParserNamespace.prototype.registerNative = function () {
        if (this.elementSet.dirty) {
            this.native.setElementTrie(this.elementSet.encodeTrie());
            this.elementSet.dirty = false;
        }
        if (this.attributeSet.dirty) {
            this.native.setAttributeTrie(this.attributeSet.encodeTrie());
            this.attributeSet.dirty = false;
        }
        return (this.native);
    };
    ParserNamespace.prototype.addElement = function (name) {
        return (this.elementSet.createToken(name, this));
    };
    ParserNamespace.prototype.addAttribute = function (name) {
        return (this.attributeSet.createToken(name, this));
    };
    return ParserNamespace;
}());
exports.ParserNamespace = ParserNamespace;
