import { Namespace } from '../Namespace';
import { ParserNamespace } from './ParserNamespace';
import { ParserConfig } from './ParserConfig';
export declare type TokenBuffer = (Token | number | string)[];
export declare const enum TokenKind {
    open = 0,
    close = 1,
    emitted = 2,
    elementEnd = 2,
    string = 3,
    number = 4,
    attributeEnd = 4,
    comment = 5,
    blank = 6,
    uri = 7,
    prefix = 8,
    element = 9,
    attribute = 10,
    other = 11,
}
export declare abstract class Token {
    constructor();
    serialize?(indent?: string, config?: ParserConfig, data?: any): string | TokenBuffer;
    serializeJson?(indent?: string, config?: ParserConfig, data?: any): any;
    kind: TokenKind;
    kindString: string;
}
export declare class SpecialToken extends Token {
    kind: TokenKind;
    kindString: string;
    constructor(kind: TokenKind, kindString: string);
    static comment: SpecialToken;
    static blank: SpecialToken;
}
export declare abstract class MemberToken extends Token {
    name: string;
    ns: Namespace;
    id: number | undefined;
    constructor(name: string, ns: Namespace, id?: number | undefined);
    abstract resolve(ns: ParserNamespace): Token;
}
export declare class ElementToken extends MemberToken {
    resolve(ns: ParserNamespace): OpenToken | CloseToken | EmittedToken | StringToken;
}
export declare class AttributeToken extends MemberToken {
    resolve(ns: ParserNamespace): OpenToken | CloseToken | EmittedToken | StringToken;
}
export declare class OpenToken extends ElementToken {
    emitted: EmittedToken;
    close: CloseToken;
}
export declare class CloseToken extends ElementToken {
}
export declare class EmittedToken extends ElementToken {
}
export declare class StringToken extends AttributeToken {
}
