"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ComplexType_1 = require("./ComplexType");
var Attribute_1 = require("./Attribute");
var Element_1 = require("./Element");
var SimpleSchema = /** @class */ (function () {
    function SimpleSchema(parserConfig, ns, spec, root) {
        if (root === void 0) { root = spec['document']; }
        this.parserConfig = parserConfig;
        this.ns = ns;
        this.typeTbl = {};
        var typeTbl = this.typeTbl;
        // SimpleType expands to a plain string without support for attributes,
        // ComplexType expands to { $: "..." } allowing parseUnknown to work.
        // const stringType = new SimpleType();
        var stringType = new ComplexType_1.ComplexType();
        typeTbl['xs:string'] = stringType;
        parserConfig.addNamespace(ns);
        // Create placeholder objects for all types.
        for (var _i = 0, _a = Object.keys(spec); _i < _a.length; _i++) {
            var typeName = _a[_i];
            typeTbl[typeName] = new ComplexType_1.ComplexType();
        }
        // Define types, using placeholders when referring to undefined types.
        for (var _b = 0, _c = Object.keys(spec); _b < _c.length; _b++) {
            var typeName = _c[_b];
            this.defineType(spec[typeName], typeTbl[typeName]);
        }
        this.document = (typeTbl['document'] || this.defineType(root));
    }
    SimpleSchema.prototype.defineType = function (spec, type) {
        if (type === void 0) { type = new ComplexType_1.ComplexType(); }
        var memberName;
        for (var _i = 0, spec_1 = spec; _i < spec_1.length; _i++) {
            var child = spec_1[_i];
            if (typeof (child) == 'string') {
                memberName = child;
                child = {};
                child[memberName] = memberName;
            }
            for (var _a = 0, _b = Object.keys(child); _a < _b.length; _a++) {
                memberName = _b[_a];
                var min = 1, max = 1;
                // Parse element or attribute name with type prefix / suffix.
                var parts = memberName.match(/(\$?)([^\[]+)(\[\])?(\?)?/);
                if (!parts)
                    continue;
                var prefix = parts[1], name_1 = parts[2], arraySuffix = parts[3], optionalSuffix = parts[4];
                // Parse type name if it differs from element/attribute name.
                if (child[memberName] != memberName) {
                    parts = child[memberName].match(/(\$?)([^\[]+)(\[\])?(\?)?/);
                    if (!parts)
                        continue;
                    // Type prefix / suffix behave identically in member and type names.
                    prefix = prefix || parts[1];
                    arraySuffix = arraySuffix || parts[3];
                    optionalSuffix = optionalSuffix || parts[4];
                }
                if (optionalSuffix)
                    min = 0;
                if (arraySuffix)
                    max = Infinity;
                var memberTypeName = parts[2];
                var memberType = this.typeTbl[memberTypeName];
                // Prefix $ marks attributes.
                if (prefix == '$') {
                    var token = this.parserConfig.getAttributeTokens(this.ns, name_1)[3 /* string */];
                    var attributeSpec = new Attribute_1.AttributeSpec(min, max);
                    var attributeMeta = new Attribute_1.AttributeMeta(token);
                    // attributeMeta.type = xsd:string
                    attributeSpec.meta = attributeMeta;
                    type.addAttribute(attributeSpec);
                }
                else if (memberType) {
                    var token = this.parserConfig.getElementTokens(this.ns, name_1)[0 /* open */];
                    var elementSpec = void 0;
                    var elementMeta = void 0;
                    if (memberType instanceof ComplexType_1.ComplexType) {
                        elementSpec = new Element_1.ElementSpec(min, max);
                        elementMeta = new Element_1.ElementMeta(token);
                    }
                    else {
                        elementSpec = new Element_1.SimpleElementSpec(min, max);
                        elementMeta = new Element_1.SimpleElementMeta(token);
                    }
                    elementMeta.type = memberType;
                    elementSpec.meta = elementMeta;
                    type.addAll(elementSpec);
                }
            }
        }
        return (type);
    };
    return SimpleSchema;
}());
exports.SimpleSchema = SimpleSchema;
