"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Patricia_1 = require("./Patricia");
var TokenSet = /** @class */ (function () {
    function TokenSet(space, parent) {
        this.space = space;
        this.dirty = true;
        this.isIndependent = !parent;
        if (parent) {
            this.tbl = parent.tbl;
            this.trie = parent.trie;
        }
        else {
            this.tbl = {};
            this.trie = new Patricia_1.Patricia();
        }
    }
    TokenSet.prototype.makeIndependent = function () {
        if (this.isIndependent)
            return;
        this.isIndependent = true;
        var tbl = {};
        for (var _i = 0, _a = Object.keys(this.tbl); _i < _a.length; _i++) {
            var key = _a[_i];
            tbl[key] = this.tbl[key];
        }
        this.tbl = tbl;
        this.trie = this.trie.clone();
    };
    TokenSet.prototype.createToken = function (name, ns) {
        var token = this.tbl[name];
        if (!token) {
            if (!this.isIndependent)
                this.makeIndependent();
            token = this.space.createToken(name, ns);
            this.tbl[name] = token;
            if (token.name) {
                this.dirty = true;
                this.trie.insertNode(token);
            }
        }
        return (token);
    };
    TokenSet.prototype.addToken = function (token) {
        if (token.name) {
            this.dirty = true;
            this.tbl[token.name] = token;
            this.trie.insertNode(token);
        }
    };
    TokenSet.prototype.encodeTrie = function () {
        return (this.trie.encode());
    };
    return TokenSet;
}());
exports.TokenSet = TokenSet;
