"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var stream = require("stream");
var Token_1 = require("../parser/Token");
var Writer_1 = require("./Writer");
var JsonWriter = /** @class */ (function (_super) {
    __extends(JsonWriter, _super);
    /** @param config Parser config passed to any custom serializers.
      * @param data Arbitrary data passed to any custom serializers. */
    function JsonWriter(config, data) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.config = config;
        _this.data = data;
        _this.state = 2 /* TEXT */;
        _this.depth = 1 /* MIN_DEPTH */;
        _this.indent = '';
        return _this;
    }
    JsonWriter.prototype.transform = function (chunk) {
        var state = this.state;
        var depth = this.depth;
        var indent = this.indent;
        var nsElement = this.nsElement;
        var buffer = chunk.buffer;
        var token;
        var member;
        var prefix;
        var serialized;
        var partList = [];
        var partNum = -1;
        var lastNum = chunk.length - 1;
        var tokenNum = -1;
        while (tokenNum < lastNum) {
            token = buffer[++tokenNum];
            if (token instanceof Token_1.Token) {
                switch (token.kind) {
                    case 0 /* open */:
                        member = token;
                        nsElement = member.ns;
                        if (nsElement.isSpecial && nsElement.defaultPrefix == '?') {
                            state = 1 /* PROCESSING */;
                        }
                        else {
                            ++depth;
                            partList[++partNum] = indent + '[ "' + member.name + '"';
                            state = 0 /* ELEMENT */;
                        }
                        indent = ',' + Writer_1.indentPattern.substr(0, depth);
                        break;
                    case 2 /* emitted */:
                        state = 2 /* TEXT */;
                        break;
                    case 1 /* close */:
                        if (state != 1 /* PROCESSING */) {
                            member = token;
                            indent = Writer_1.indentPattern.substr(0, --depth);
                            if (state == 2 /* TEXT */) {
                                partList[++partNum] = indent + ']';
                            }
                            else {
                                partList[++partNum] = ' ]';
                            }
                            indent = ',' + indent;
                        }
                        state = 2 /* TEXT */;
                        break;
                    case 3 /* string */:
                        member = token;
                        partList[++partNum] = ', [ "$' + member.name + '"';
                        break;
                    case 5 /* comment */:
                        state = 4 /* COMMENT */;
                        break;
                    case 11 /* other */:
                        if (token.serializeJson) {
                            serialized = token.serializeJson(indent, this.config, this.data);
                            if (typeof (serialized) != 'string')
                                serialized = JSON.stringify(serialized);
                            partList[++partNum] = indent + serialized;
                            state = 3 /* AFTER_TEXT */;
                        }
                        break;
                }
            }
            else {
                switch (state) {
                    case 2 /* TEXT */:
                        partList[++partNum] = ', [ "$", ' + JSON.stringify(token) + ' ]';
                        state = 3 /* AFTER_TEXT */;
                        break;
                    case 0 /* ELEMENT */:
                    case 1 /* PROCESSING */:
                        partList[++partNum] = ', ' + JSON.stringify(token) + ' ]';
                        break;
                    case 4 /* COMMENT */:
                        break;
                }
            }
        }
        this.state = state;
        this.depth = depth;
        this.indent = indent;
        this.nsElement = nsElement;
        return (partList);
    };
    JsonWriter.prototype._transform = function (chunk, enc, flush) {
        if (!chunk) {
            flush(null, '');
            return;
        }
        var partList = this.transform(chunk);
        flush(null, partList.join(''));
    };
    JsonWriter.prototype._flush = function (flush) {
        flush(null, '\n');
    };
    return JsonWriter;
}(stream.Transform));
exports.JsonWriter = JsonWriter;
