"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var stream = require("stream");
var Namespace_1 = require("../Namespace");
var TokenChunk_1 = require("../parser/TokenChunk");
var Token_1 = require("../parser/Token");
exports.indentPattern = '\n' + new Array(256 /* MAX_DEPTH */).join('\t');
var Writer = /** @class */ (function (_super) {
    __extends(Writer, _super);
    /** @param config Parser config passed to any custom serializers.
      * @param data Arbitrary data passed to any custom serializers. */
    function Writer(config, data) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.config = config;
        _this.data = data;
        _this.chunkCount = 0;
        _this.state = 2 /* TEXT */;
        _this.depth = 1 /* MIN_DEPTH */;
        _this.indent = '';
        _this.prefixList = [];
        _this.prefixTbl = {};
        _this.xmlnsDefinitions = '';
        return _this;
    }
    Writer.prototype.transform = function (chunk, partList) {
        var prefixList = this.prefixList;
        var chunkCount = this.chunkCount++;
        var buffer;
        var state = this.state;
        var depth = this.depth;
        var indent = this.indent;
        var nsElement = this.nsElement;
        var token;
        var member;
        var prefix;
        var serialized;
        var partNum = partList.length - 1;
        var lastNum = chunk.length - 1;
        var tokenNum = -1;
        var namespaceList;
        if (typeof (chunk) == 'string') {
            partList.push(chunk);
            return (partList);
        }
        else if (chunk instanceof TokenChunk_1.TokenChunk) {
            buffer = chunk.buffer;
            namespaceList = chunk.namespaceList;
        }
        else {
            buffer = chunk;
        }
        if (!chunkCount) {
            if (!namespaceList) {
                namespaceList = [];
                while (tokenNum < lastNum) {
                    token = buffer[++tokenNum];
                    if (token instanceof Token_1.MemberToken) {
                        namespaceList[token.ns.id] = token.ns;
                    }
                }
                tokenNum = -1;
            }
            this.copyPrefixes(namespaceList);
        }
        while (tokenNum < lastNum) {
            token = buffer[++tokenNum];
            if (token instanceof Token_1.Token) {
                switch (token.kind) {
                    case 0 /* open */:
                        member = token;
                        nsElement = member.ns;
                        partList[++partNum] = indent + '<' + prefixList[nsElement.id] + member.name;
                        if (nsElement.isSpecial && nsElement.defaultPrefix == '?') {
                            state = 1 /* PROCESSING */;
                        }
                        else {
                            if (depth++ == 1 /* MIN_DEPTH */)
                                partList[++partNum] = this.xmlnsDefinitions;
                            state = 0 /* ELEMENT */;
                        }
                        indent = exports.indentPattern.substr(0, depth);
                        break;
                    case 2 /* emitted */:
                        partList[++partNum] = '>';
                        state = 2 /* TEXT */;
                        break;
                    case 1 /* close */:
                        if (state == 1 /* PROCESSING */) {
                            partList[++partNum] = '?>';
                        }
                        else {
                            member = token;
                            indent = exports.indentPattern.substr(0, --depth);
                            if (state == 0 /* ELEMENT */) {
                                partList[++partNum] = '/>';
                            }
                            else {
                                if (state != 3 /* AFTER_TEXT */)
                                    partList[++partNum] = indent;
                                partList[++partNum] = '</' + prefixList[member.ns.id] + member.name + '>';
                            }
                        }
                        state = 2 /* TEXT */;
                        break;
                    case 3 /* string */:
                        member = token;
                        // Omit prefixes for attributes in the same namespace
                        // as their parent element.
                        if (member.ns == nsElement)
                            prefix = '';
                        else
                            prefix = prefixList[member.ns.id];
                        partList[++partNum] = ' ' + prefix + member.name + '=';
                        break;
                    case 5 /* comment */:
                        state = 4 /* COMMENT */;
                        break;
                    case 11 /* other */:
                        if (token.serialize) {
                            serialized = token.serialize(indent, this.config, this.data);
                            if (typeof (serialized) == 'string') {
                                partList[++partNum] = serialized;
                                state = 3 /* AFTER_TEXT */;
                            }
                            else {
                                this.state = state;
                                this.depth = depth;
                                this.indent = indent;
                                this.transform(TokenChunk_1.TokenChunk.allocate(serialized), partList);
                                partNum = partList.length - 1;
                            }
                        }
                        break;
                }
            }
            else {
                switch (state) {
                    case 2 /* TEXT */:
                        partList[++partNum] = '' + token;
                        state = 3 /* AFTER_TEXT */;
                        break;
                    case 0 /* ELEMENT */:
                    case 1 /* PROCESSING */:
                        partList[++partNum] = '"' + token + '"';
                        break;
                    case 4 /* COMMENT */:
                        partList[++partNum] = indent + '<!--' + token;
                        break;
                }
            }
        }
        this.state = state;
        this.depth = depth;
        this.indent = indent;
        this.nsElement = nsElement;
        if (chunk instanceof TokenChunk_1.TokenChunk)
            chunk.free();
        return (partList);
    };
    Writer.prototype._transform = function (chunk, enc, flush) {
        if (!chunk) {
            flush(null, '');
            return;
        }
        var partList = [];
        if (!this.chunkCount) {
            var token = chunk instanceof TokenChunk_1.TokenChunk ? chunk.buffer[0] : chunk[0];
            if (!(token instanceof Token_1.Token) ||
                token.kind != 0 /* open */ ||
                token.ns != Namespace_1.Namespace.processing ||
                token.name != 'xml') {
                partList.push('<?xml version="1.0" encoding="utf-8"?>\n');
            }
        }
        this.transform(chunk, partList);
        flush(null, partList.join(''));
    };
    Writer.prototype._flush = function (flush) {
        flush(null, '\n');
    };
    Writer.prototype.copyPrefixes = function (namespaceList) {
        var prefixTbl = this.prefixTbl;
        var prefixList = this.prefixList;
        var prefix;
        var ns;
        // Add a number to distinguish between duplicate prefix names.
        for (var i = 0; i < namespaceList.length; ++i) {
            ns = namespaceList[i];
            if (!ns)
                continue;
            prefix = ns.defaultPrefix;
            if (!prefix && !ns.isSpecial)
                continue;
            if (prefixTbl[prefix]) {
                var j_1 = 1;
                do {
                    prefix = ns.defaultPrefix + (++j_1);
                } while (prefixTbl[prefix]);
            }
            prefixList[i] = prefix;
            prefixTbl[prefix] = i + 1;
        }
        var j = 0;
        // Name all unnamed prefixes with "p" and a sequence number.
        for (var i = 0; i < namespaceList.length; ++i) {
            ns = namespaceList[i];
            if (!ns)
                continue;
            prefix = ns.defaultPrefix;
            if (prefix || ns.isSpecial)
                continue;
            do {
                prefix = 'p' + (++j);
            } while (prefixTbl[prefix]);
            prefixList[i] = prefix;
            prefixTbl[prefix] = i + 1;
        }
        var definitionList = [];
        for (var i = 0; i < namespaceList.length; ++i) {
            ns = namespaceList[i];
            prefix = prefixList[i];
            if (!prefix || !ns || ns.isSpecial)
                continue;
            if (prefix != 'xml')
                definitionList.push(' xmlns:' + prefix + '="' + ns.uri + '"');
            this.prefixList[i] = prefix + ':';
        }
        this.xmlnsDefinitions = definitionList.join('');
    };
    return Writer;
}(stream.Transform));
exports.Writer = Writer;
